import javax.swing.*;
import javax.swing.border.LineBorder;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Method;

public class FormArmoredCarConfig extends JDialog{
    private JPanel mainPanel;
    private JButton buttonOk;
    private JButton buttonCancel;
    private JSpinner numericUpDownSpeed;
    private JSpinner numericUpDownWeight;
    private JCheckBox checkBoxAMachineGun;
    private JCheckBox checkBoxTowerWeapon;
    private JLabel labelSpeed;
    private JLabel labelWeight;
    private JLabel labelParameters;
    private JLabel labelSimpleObject;
    private JLabel labelModifiedObject;
    private JLabel labelBaseColor;
    private JLabel labelDopColor;
    private JPanel drawPanel;
    private JPanel colorsPanel;
    private JLabel labelSimpleCaterpillar;
    private JLabel labelDoubleCaterpillar;
    private JLabel labelCrossCaterpillar;
    private JSpinner numericUpDownCount;
    private JLabel labelCatCounts;

    private DrawingArmoredCar armoredCar = null;
    private Delegator<DrawingArmoredCar> delegator;

    public FormArmoredCarConfig() {
        setTitle("Создание объекта");
        setBounds(100, 100, 800, 300);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        setContentPane(mainPanel);
        setVisible(true);
        labelSimpleObject.setBorder(new LineBorder(Color.BLACK));
        labelModifiedObject.setBorder(new LineBorder(Color.BLACK));
        labelBaseColor.setBorder(new LineBorder(Color.BLACK));
        labelDopColor.setBorder(new LineBorder(Color.BLACK));
        labelSimpleCaterpillar.setBorder(new LineBorder(Color.BLACK));
        labelDoubleCaterpillar.setBorder(new LineBorder(Color.BLACK));
        labelCrossCaterpillar.setBorder(new LineBorder(Color.BLACK));

        var mouseAdapter = new DragMouseAdapter();

        labelSimpleObject.addMouseListener(mouseAdapter);
        labelSimpleObject.setTransferHandler(new TransferHandler("text"));

        labelModifiedObject.addMouseListener(mouseAdapter);
        labelModifiedObject.setTransferHandler(new TransferHandler("text"));

        labelSimpleCaterpillar.addMouseListener(mouseAdapter);
        labelDoubleCaterpillar.addMouseListener(mouseAdapter);
        labelCrossCaterpillar.addMouseListener(mouseAdapter);

        labelSimpleCaterpillar.setTransferHandler(new TransferHandler("text"));
        labelDoubleCaterpillar.setTransferHandler(new TransferHandler("text"));
        labelCrossCaterpillar.setTransferHandler(new TransferHandler("text"));

        drawPanel.setBackground(null);
        colorsPanel.setBorder(BorderFactory.createTitledBorder("Цвета"));
        colorsPanel.setLayout(new GridLayout(2, 4));
        colorsPanel.setBackground(null);
        setColors();
        buttonCancel.addActionListener((e) -> dispose());
        new DrawPanelDropTargetListener(drawPanel);
        new LabelColorDropTargetListener(labelBaseColor);
        new LabelColorDropTargetListener(labelDopColor);


        buttonOk.addActionListener(e -> {
            if (delegator != null)
                delegator.invoke(armoredCar);
            dispose();
        });
    }

    private class DragMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            var c = (JComponent) e.getSource();
            var handler = c.getTransferHandler();
            handler.exportAsDrag(c, e, TransferHandler.COPY);
        }
    }

    private class LabelColorDropTargetListener extends DropTargetAdapter{
        private final DropTarget dropTarget;
        private final JLabel label;

        public LabelColorDropTargetListener(JLabel label) {
            this.label = label;
            dropTarget = new DropTarget(label, DnDConstants.ACTION_COPY,
                    this, true, null);
        }
        @Override
        public void drop(DropTargetDropEvent event) {
            try {
                var tr = event.getTransferable();
                var dataFlavors = tr.getTransferDataFlavors();
                String s = (String) tr.getTransferData(dataFlavors[0]);
                Color color = Color.decode(s);
                switch (label.getText()) {
                    case "Цвет": {
                        if (armoredCar != null) {
                            armoredCar.SetColor(color);
                        }
                        break;
                    }
                    case "Доп. Цвет": {
                        if (armoredCar != null) {
                            if (armoredCar instanceof DrawingTank tank) {
                                tank.SetDopColor(color);
                            }
                        }
                        break;
                    }
                }
                drawPanel.repaint();
            } catch (Exception e) {
                event.rejectDrop();
            }
        }
    }

    private class DrawPanelDropTargetListener extends DropTargetAdapter {

        private final DropTarget dropTarget;
        private final JPanel panel;

        public DrawPanelDropTargetListener(JPanel panel) {
            this.panel = panel;

            dropTarget = new DropTarget(panel, DnDConstants.ACTION_COPY,
                    this, true, null);
        }


        public void drop(DropTargetDropEvent event) {
            try {
                var tr = event.getTransferable();
                var dataFlavors = tr.getTransferDataFlavors();
                String drawing_type = (String) tr.getTransferData(dataFlavors[0]);
                event.acceptDrop(DnDConstants.ACTION_COPY);
                switch (drawing_type) {
                    case "Простой": {
                        armoredCar = new DrawingArmoredCar((int) numericUpDownSpeed.getValue(), (int) numericUpDownWeight.getValue(), Color.WHITE);
                        IDrawingCaterpillar drawingCaterpillar = armoredCar.GetCaterpillars();
                        drawingCaterpillar.setNumRinks((int)numericUpDownCount.getValue());
                        break;
                    }
                    case "Продвинутый": {
                        armoredCar = new DrawingTank((int) numericUpDownSpeed.getValue(), (int) numericUpDownWeight.getValue(), Color.WHITE, Color.BLACK,
                                checkBoxTowerWeapon.isSelected(), checkBoxAMachineGun.isSelected());
                        IDrawingCaterpillar drawingCaterpillar = armoredCar.GetCaterpillars();
                        drawingCaterpillar.setNumRinks((int)numericUpDownCount.getValue());
                        break;
                    }
                    case "Обычные": {
                        if (armoredCar != null) {
                            IDrawingCaterpillar drawingCaterpillar = new DrawingCaterpillar(armoredCar.armoredCar.getBodyColor());
                            drawingCaterpillar.setNumRinks((int)numericUpDownCount.getValue());
                            armoredCar.SetCaterpillar(drawingCaterpillar);
                        }
                        break;
                    }
                    case "Двойные": {
                        if (armoredCar != null) {
                            IDrawingCaterpillar drawingCaterpillar = new DrawingDoubleCaterpillar(armoredCar.armoredCar.getBodyColor());
                            drawingCaterpillar.setNumRinks((int)numericUpDownCount.getValue());
                            armoredCar.SetCaterpillar(drawingCaterpillar);
                        }
                        break;
                    }
                    case "Крестики": {
                        if (armoredCar != null) {
                            IDrawingCaterpillar drawingCaterpillar = new DrawingCrossCaterpillar(armoredCar.armoredCar.getBodyColor());
                            drawingCaterpillar.setNumRinks((int)numericUpDownCount.getValue());
                            armoredCar.SetCaterpillar(drawingCaterpillar);
                        }
                        break;
                    }
                 }
                event.dropComplete(true);
                drawPanel.repaint();
            } catch (Exception e) {
                e.printStackTrace();
                event.rejectDrop();
            }
        }
    }

    private void createUIComponents() {
        drawPanel = new JPanel() {
            @Override
            public void paintComponent(Graphics g) {
                super.paintComponent(g);
                if (armoredCar != null) {
                    Graphics2D g2d = (Graphics2D) g;
                    armoredCar.SetPosition(drawPanel.getWidth() / 4, drawPanel.getHeight() / 4, drawPanel.getWidth(), drawPanel.getHeight());
                    armoredCar.DrawTransport(g2d);
                }
                super.repaint();
            }
        };
        numericUpDownCount = new JSpinner(new SpinnerNumberModel(4, 4, 6, 1));
        numericUpDownSpeed = new JSpinner(new SpinnerNumberModel(100, 100, 1000, 1));
        numericUpDownWeight = new JSpinner(new SpinnerNumberModel(100, 100, 1000, 1));
    }

    private void setColors() {
        Color[] colors = new Color[8];
        colors[0] = Color.RED;
        colors[1] = Color.GREEN;
        colors[2] = Color.BLUE;
        colors[3] = Color.YELLOW;
        colors[4] = Color.WHITE;
        colors[5] = Color.GRAY;
        colors[6] = Color.BLACK;
        colors[7] = Color.MAGENTA;

        int i = 0;
        int width = 40;
        for (Color color : colors) {
            JButton btn = new JButton();
            btn.setBackground(color);
            btn.setEnabled(false);
            int x = i % 4;
            int y = i / 4;

            btn.setBounds(x * width + 10, y * width + 10, width, width);
            btn.setText(Integer.toString(color.getRGB()));
            btn.setFont(new Font("Arial", Font.PLAIN, 0));
            btn.setTransferHandler(new TransferHandler("text"));
            btn.addMouseListener(new DragMouseAdapter());
            colorsPanel.add(btn);
            i++;
        }
    }

    public void addEvent(Method event, Object consumer) {
        if (delegator == null)
        {
            delegator = new Delegator<>();
        }
        delegator.add(event, consumer);
    }
}
