import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Queue;

public class FormMapWithArmoredCars extends JFrame{
    private JButton buttonAddArmoredCar;
    private JButton buttonRemoveArmoredCar;
    private JButton buttonShowStorage;
    private JButton buttonShowOnMap;
    private JTextField maskedTextBoxPosition;
    private JComboBox mapSelectorComboBox;
    private JPanel drawPanel;
    private JButton buttonUp;
    private JButton buttonDown;
    private JButton buttonRight;
    private JButton buttonLeft;
    private JPanel mainPanel;
    private JPanel btnPanel;
    private JTextField textBoxNewMapName;
    private JButton AddMapButton;
    private JButton DeleteMapButton;
    private JList listBoxMaps;
    private JButton lookDeletedButton;

    private Image img;

    private final HashMap<String, AbstractMap> _mapsDict = new HashMap<>() {{
        put("Простая карта", new SimpleMap());
        put("Карта 1", new MyMapLabyrinth());
        put("Карта 2", new MyMapWooden());
    }};

    private final MapsCollection _mapsCollection;
    private final Queue<DrawingObjectArmoredCar> queue = new LinkedList<>();

    public FormMapWithArmoredCars() {
        super("Карта с набором объектов");
        setBounds(100, 100, 1000, 700);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        mapSelectorComboBox.removeAllItems();
        for (var elem : _mapsDict.keySet())
        {
            mapSelectorComboBox.addItem(elem);
        }

        buttonAddArmoredCar.addActionListener(e -> {
            var form = new FormArmoredCarConfig();
            try {
                form.addEvent(FormMapWithArmoredCars.class.getDeclaredMethod("AddArmoredCar", DrawingArmoredCar.class), this);
            } catch (NoSuchMethodException noSuchMethodException) {
                noSuchMethodException.printStackTrace();
            }
        });
        buttonRemoveArmoredCar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (maskedTextBoxPosition.getText().equals(""))
                {
                    return;
                }
                int res = JOptionPane.showConfirmDialog (null,"Удалить объект?",
                        "Удаление", JOptionPane.YES_NO_OPTION);
                if (res == JOptionPane.YES_OPTION) {
                    int pos = Integer.parseInt(maskedTextBoxPosition.getText());
                    DrawingObjectArmoredCar deletedArmoredCar = (DrawingObjectArmoredCar) _mapsCollection.get((String) listBoxMaps.getSelectedValue(), pos);
                    if (_mapsCollection.get((String) listBoxMaps.getSelectedValue()).remove(pos) != null)
                    {
                        queue.add(deletedArmoredCar);
                        JOptionPane.showMessageDialog(null, "Объект удален");
                        img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).ShowSet();
                    }
                    else
                    {
                        JOptionPane.showMessageDialog(null, "Не удалось удалить объект");
                    }
                }
            }
        });
        buttonShowStorage.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapsCollection.get((String) listBoxMaps.getSelectedValue()) == null)
                {
                    return;
                }
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).ShowSet();
                repaint();
            }
        });

        buttonShowOnMap.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapsCollection.get((String) listBoxMaps.getSelectedValue()) == null)
                {
                    return;
                }
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).ShowOnMap();
                repaint();
            }
        });

        setContentPane(mainPanel);
        setVisible(true);
        buttonLeft.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).MoveObject(Direction.Left);
                repaint();
            }
        });
        buttonUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).MoveObject(Direction.Up);
                repaint();
            }
        });
        buttonDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).MoveObject(Direction.Down);
                repaint();
            }
        });
        buttonRight.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).MoveObject(Direction.Right);
                repaint();
            }
        });
        AddMapButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (mapSelectorComboBox.getSelectedIndex() == -1 || textBoxNewMapName.getText().equals(""))
                {
                    JOptionPane.showMessageDialog(null, "Не все данные заполнены", "Ошибка", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (!_mapsDict.containsKey((String) mapSelectorComboBox.getSelectedItem()))
                {
                    JOptionPane.showMessageDialog(null, "Нет такой карты", "Ошибка", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                _mapsCollection.AddMap(textBoxNewMapName.getText(), _mapsDict.get((String) mapSelectorComboBox.getSelectedItem()));
                ReloadMaps();
            }
        });
        _mapsCollection = new MapsCollection(drawPanel.getWidth(), drawPanel.getHeight());

        listBoxMaps.addListSelectionListener(e -> {
            if (listBoxMaps.getSelectedValue() != null) {
                img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).ShowSet();
                repaint();
            }
        });

        DeleteMapButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                if (JOptionPane.showConfirmDialog(null, "Удалить карту" +
                        listBoxMaps.getSelectedValue()+ "?", "Удаление", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
                {
                    _mapsCollection.DelMap((String) listBoxMaps.getSelectedValue());
                    ReloadMaps();
                }
            }
        });
        lookDeletedButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {

                if (!queue.isEmpty()) {
                    FormArmoredCar form = new FormArmoredCar();
                    form.setArmoredCar(queue.poll().getArmoredCar());
                }
                else
                    JOptionPane.showMessageDialog(null, "Нет удаленных объектов");
            }
        });
    }

    private void createUIComponents() {
        drawPanel = new JPanel() {
            @Override
            public void paintComponent(Graphics g) {
                super.paintComponent(g);
                g.drawImage(img, 0, 0, this);
                super.repaint();
            }
        };
    }

    private void ReloadMaps() {
        int index = listBoxMaps.getSelectedIndex();
        listBoxMaps.setListData(_mapsCollection.getKeys().toArray(new Object[0]));
        if (listBoxMaps.getModel().getSize() > 0 && (index == -1 || index >=
                listBoxMaps.getModel().getSize()))
        {
            listBoxMaps.setSelectedIndex(0);
        }
        else if (listBoxMaps.getModel().getSize() > 0 && index > -1 && index <
                listBoxMaps.getModel().getSize())
        {
            listBoxMaps.setSelectedIndex(index);
        }
    }

    public void AddArmoredCar(DrawingArmoredCar drawingArmoredCar) {
        if (listBoxMaps.getSelectedIndex() == -1)
        {
            return;
        }
        DrawingObjectArmoredCar armoredCar = new DrawingObjectArmoredCar(drawingArmoredCar);
        if (_mapsCollection.get((String)listBoxMaps.getSelectedValue()).add(armoredCar) > -1)
        {
            JOptionPane.showMessageDialog(null, "Объект добавлен");
            img = _mapsCollection.get((String) listBoxMaps.getSelectedValue()).ShowSet();
            repaint();
        }
        else
        {
            JOptionPane.showMessageDialog(null, "Не удалось добавить объект");
        }
    }
}
