import java.awt.*;
import java.awt.image.BufferedImage;

public class MapWithSetArmoredCarsGeneric<T extends IDrawingObject, U extends AbstractMap> {

        private int _pictureWidth;

        private int _pictureHeight;

        private int _placeSizeWidth = 210;

        private int _placeSizeHeight = 110;

        private SetArmoredCarsGeneric<T> _setCars;

        private U _map;

        public MapWithSetArmoredCarsGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setCars = new SetArmoredCarsGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }

        public int add(T car)
        {
            return _setCars.Insert(car);
        }

        public  T remove(int position)
        {
            return _setCars.Remove(position);
        }

        public Image ShowSet()
        {
            Image bmp = new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_BGR);
            Graphics gr = bmp.getGraphics();
            DrawBackground(gr);
            DrawArmoredCars(gr);
            return bmp;
        }

        public Image ShowOnMap()
        {
            Shaking();
            for (var armoredCar : _setCars.GetArmoredCars())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, armoredCar);
            }
            return new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_BGR);
        }

        public Image MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_BGR);
        }

        private void Shaking()
        {
            int j = _setCars.getCount() - 1;
            for (int i = 0; i < _setCars.getCount(); i++)
            {
                if (_setCars.Get(i) == null)
                {
                    for (; j > i; j--)
                    {
                        var car = _setCars.Get(j);
                        if (car != null)
                        {
                            _setCars.Insert(car, i);
                            _setCars.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }

        private void DrawBackground(Graphics g)
        {
            Color colorRoad = Color.GRAY;
            Color colorBox = new Color(110, 69, 19, 255);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight; j++)
                {
                    g.setColor(colorRoad);
                    g.fillRect(i * _placeSizeWidth, j * _placeSizeHeight, _placeSizeWidth, _placeSizeHeight);

                    g.setColor(colorBox);
                    int boxSize = _placeSizeWidth / 7;
                    g.fillRect(i * _placeSizeWidth + _placeSizeWidth / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5, boxSize, boxSize);

                    g.fillRect(i * _placeSizeWidth + _placeSizeWidth / 2 - boxSize / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5 + boxSize, boxSize, boxSize);

                    g.fillRect(i * _placeSizeWidth + _placeSizeWidth / 2 + boxSize / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5 + boxSize, boxSize, boxSize);

                    g.setColor(Color.BLACK);
                    g.drawLine(i * _placeSizeWidth, j * _placeSizeHeight,
                            i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);

                    g.drawRect(i * _placeSizeWidth + _placeSizeWidth / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5, boxSize, boxSize);

                    g.drawRect( i * _placeSizeWidth + _placeSizeWidth / 2 - boxSize / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5 + boxSize, boxSize, boxSize);

                    g.drawRect( i * _placeSizeWidth + _placeSizeWidth / 2 + boxSize / 2,
                            j * _placeSizeHeight + _placeSizeHeight / 5 + boxSize, boxSize, boxSize);
                }
                g.drawLine(i * _placeSizeWidth, 0, i * _placeSizeWidth,
                        (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }

        private void DrawArmoredCars(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int i = 0;
            for (var armoredCar : _setCars.GetArmoredCars())
            {
                armoredCar.SetObject(i % width * _placeSizeWidth + 5, i / width * _placeSizeHeight + 10, _pictureWidth, _pictureHeight);
                armoredCar.DrawingObject((Graphics2D) g);
                i++;
            }
        }

        public SetArmoredCarsGeneric<T> get_setCars() {
            return _setCars;
        }

        public String GetData(char separatorType, char separatorData)
        {
            String data = String.format("%s%s", _map.getClass().getName(), separatorType);
            for (var armoredCar : _setCars.GetArmoredCars())
            {
                data += String.format("%s%s", armoredCar.GetInfo(), separatorData);
            }
            return data;
        }

        public void LoadData(String[] records)
        {
            for(var rec : records)
            {
                _setCars.Insert((T)DrawingObjectArmoredCar.Create(rec));
            }
        }


}
