import java.io.*;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class MapsCollection {
    public HashMap<String, MapWithSetArmoredCarsGeneric<IDrawingObject, AbstractMap>> _mapStorages;
    private int _pictureWidth;
    private int _pictureHeight;

    private final char separatorDict = '#';
    private final char separatorData = ';';

    public MapsCollection(int pictureWidth, int pictureHeight)
    {
        _mapStorages = new HashMap<>();
        _pictureWidth = pictureWidth;
        _pictureHeight = pictureHeight;
    }

    public Set<String> getKeys() {
        return _mapStorages.keySet();
    }

    public void AddMap(String name, AbstractMap map)
    {
        if (!getKeys().contains(name))
            _mapStorages.put(name, new MapWithSetArmoredCarsGeneric<>(_pictureWidth, _pictureHeight, map));
    }

    public void DelMap(String name)
    {
        if (getKeys().contains(name))
            _mapStorages.remove(name);
    }

    public MapWithSetArmoredCarsGeneric<IDrawingObject, AbstractMap> get(String ind)
    {
        if (getKeys().contains(ind))
            return _mapStorages.get(ind);
        return null;
    }
    // дополнительное задание: метод "индексатор" с двумя параметрами
    public IDrawingObject get(String ind1, int pos) {
        if (_mapStorages.containsKey(ind1))
            return _mapStorages.get(ind1).get_setCars().Get(pos);
        return null;
    }

    public boolean SaveData(String filename)
    {
        File f = new File(filename);
        if (f.exists())
        {
            f.delete();
        }
        try (BufferedWriter writter = new BufferedWriter(new FileWriter(filename))) {
            writter.write("MapsCollection");
            writter.newLine();
            for (var key : _mapStorages.keySet())
            {
                writter.write(String.format("%s%s%s", key, separatorDict, _mapStorages.get(key).GetData(separatorDict, separatorData)));
            }
        }
        catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public boolean LoadData(String filename) {
        File f = new File(filename);
        if (!f.exists()) {
            return false;
        }
        try (BufferedReader reader = new BufferedReader(new FileReader(filename))) {
            if (!reader.readLine().contains("MapsCollection"))
                return false;
            String line;
            while ((line = reader.readLine()) != null) {
                var elem = line.split(String.valueOf(separatorDict));
                AbstractMap map = null;
                switch (elem[1]) {
                    case "SimpleMap":
                        map = new SimpleMap();
                        break;
                    case "MyMapLabirinth":
                        map = new MyMapLabyrinth();
                        break;
                    case "MyMapWooden":
                        map = new MyMapWooden();
                        break;
                }
                _mapStorages.put(elem[0], new MapWithSetArmoredCarsGeneric<>(_pictureWidth, _pictureHeight, map));
                _mapStorages.get(elem[0]).LoadData(elem[2].split(String.valueOf(separatorData)));
            }
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }
}
